var grammar = {
	'value': [
		['object'],
		['array'],
		['string'],
		['number'],
		['atom'],
	],
	'object': [
		['{', 'string', ':', 'value', 'object-tail']
	],
	'object-tail': [
		[',', 'string', ':', 'value', 'object-tail'],
		['}']
	],
	'array': [
		['[', 'value', 'array-tail']
	],
	'array-tail': [
		[',', 'value', 'array-tail'],
		[']']
	]
};

var can_expand = function(node, token) {
	if (token.type === node) {
		return true;
	}
	if (grammar[node] === undefined) {
		return false
		// errors thrown from here??
	}
	for (var i = 0; i < grammar[node].length; i++) {
		if (can_expand(grammar[node][i][0], token)) {
			return true;
		}
	}
	return false;
}

var guess = function(guess_node, tokens, start) {
	if (tokens[start].type === guess_node) { // token already matches
		return true
	}
	if (grammar[guess_node] === undefined) { // token doesn't match and can't be expanded
		return false
	}
	for (var i = 0; i < grammar[guess_node].length; i++) { // look for matching expansion
		var posibility = grammar[guess_node][i];
		//console.log('expandable??', posibility[0], tokens[start], can_expand(posibility[0], tokens[start]));
		if (can_expand(posibility[0], tokens[start])) { // assume this expansion if possible
			var subtree = [];
			for (var j = 0; j < posibility.length; j++) {
				var node = posibility[j];
				//console.log('guessing', node, start, j, tokens);
				if (guess(node, tokens, start + j)) {
					subtree.push(tokens[start + j]);
				} else {
					return false;
				}
			}
			tokens.splice(start, subtree.length, {type:guess_node, tree:subtree});
			return true;
		}
	}
	return false;
}

exports.parse = function(tokens) {
	if (guess('value', tokens, 0)) {
		return tokens;
	} else {
		throw 'parse error';
	}
}